package cn.com.qmth.examcloud.exchange.outer.api.provider;

import java.io.File;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.disk.DiskFileItem;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.ZipUtil;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.exchange.outer.service.FaceppService;

/**
 * 学生照片接口
 * 
 * @author chenken
 * @date 2018年4月3日 下午3:58:31
 * @company QMTH
 * @description FaceppController.java
 */
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/facepp")
public class FaceppController extends ControllerSupport {

	/**
	 * ZIP最大50M
	 */
	private static final int ZIP_MAX_SIZE = 50;

	/**
	 * 每张照片最大尺寸 500KB
	 */
	private static final int MAX_SIZE = 500;

	@Autowired
	private FaceppService faceppService;

	/**
	 * 重构
	 *
	 * @author WANGWEI
	 * @param file
	 * @throws Exception
	 */
	@PostMapping("/add")
	public void add(@RequestParam CommonsMultipartFile file) throws Exception {
		User accessUser = getAccessUser();

		log.debug("fileName =" + file.getOriginalFilename());
		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		String filename = file.getOriginalFilename();
		faceppService.uploadPhoto(accessUser, filename, storeLocation);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param file
	 * @param request
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/import")
	public List<Map<String, String>> batchAll(@RequestParam CommonsMultipartFile file,
			HttpServletRequest request) throws Exception {
		User accessUser = getAccessUser();

		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		String filename = file.getOriginalFilename();

		if (!filename.endsWith(".zip")) {
			throw new StatusException("EX-620001", "文件格式不正确，必须是zip格式的压缩包");
		}
		// 文件大小限制
		if (file.getSize() > ZIP_MAX_SIZE * 1024 * 1024) {
			throw new StatusException("EX-620002", "文件大小超过50M,请分批导入");
		}

		List<Map<String, String>> ret = Lists.newArrayList();

		String tempDirPath = storeLocation.getCanonicalPath() + File.separator
				+ System.currentTimeMillis();
		File tempDir = new File(tempDirPath);
		try {
			ZipUtil.unzip(storeLocation, new File(tempDirPath));
		} catch (Exception e) {
			throw new StatusException("EX-620003", "zip文件损坏");
		}

		File[] files = tempDir.listFiles();

		if (null == files) {
			throw new StatusException("EX-620004", "zip文件为空");
		}

		for (File f : files) {
			Map<String, String> map = Maps.newHashMap();

			if (f.length() > MAX_SIZE * 1024) {
				map.put("statusCode", "EX-620005");
				map.put("statusDesc", "文件大小超过500KB");
				ret.add(map);
				continue;
			}
			try {
				map.put("file", f.getName());
				faceppService.uploadPhoto(accessUser, f.getName(), f);
				map.put("statusCode", "200");
				map.put("statusDesc", "成功");
			} catch (StatusException e) {
				map.put("statusCode", e.getCode());
				map.put("statusDesc", e.getDesc());
			} catch (Exception e) {
				map.put("statusCode", "系统异常");
				map.put("statusDesc", e.getCause().getMessage());
			}
			ret.add(map);
		}

		tempDir.delete();

		return ret;
	}

}
