package cn.com.qmth.examcloud.exchange.outer.api.provider;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.HttpClientUtil;
import cn.com.qmth.examcloud.commons.base.util.UrlUtil;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.InsertOrUpdateStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.InsertOrUpdateStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.StudentOuterService;
import cn.com.qmth.examcloud.exchange.outer.api.request.SaveStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.SaveStudentResp;
import cn.com.qmth.examcloud.exchange.outer.service.UpyunService;
import io.swagger.annotations.ApiOperation;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年6月29日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class StudentOuterServiceProvider extends ControllerSupport implements StudentOuterService {

	private static final long serialVersionUID = -8252740695302763716L;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	UpyunService upyunService;

	@ApiOperation(value = "保存学生信息")
	@PostMapping("saveStudent")
	@Override
	public SaveStudentResp saveStudent(@RequestBody SaveStudentReq req) {
		String identityNumber = req.getIdentityNumber();
		Long rootOrgId = req.getRootOrgId();

		String photoPath = req.getPhotoPath();
		if (StringUtils.isNotBlank(photoPath)) {
			if (photoPath.startsWith("http")) {
				byte[] bs = HttpClientUtil.get(photoPath);

				int lastIndexOf = photoPath.lastIndexOf(".");
				if (0 > lastIndexOf) {
					throw new StatusException("EX-100002",
							"photoPath is not end with photo file suffix.");
				}
				String fileSuffix = photoPath.substring(lastIndexOf);
				photoPath = upyunService.saveStudentPhoto(UrlUtil.encode(identityNumber), rootOrgId,
						fileSuffix, bs);
			}
		}

		InsertOrUpdateStudentReq request = new InsertOrUpdateStudentReq();
		request.setIdentityNumber(req.getIdentityNumber());
		request.setName(req.getName());
		request.setOrgCode(req.getOrgCode());
		request.setOrgId(req.getOrgId());
		request.setOrgName(req.getOrgName());
		request.setPhoneNumber(req.getPhoneNumber());
		request.setRemark(req.getRemark());
		request.setRootOrgId(req.getRootOrgId());
		request.setStudentCode(req.getStudentCode());
		request.setPhotoPath(photoPath);

		InsertOrUpdateStudentResp response = studentCloudService.insertOrUpdateStudent(request);
		SaveStudentResp resp = new SaveStudentResp();
		resp.setOrgId(response.getOrgId());
		resp.setOrgName(response.getOrgName());
		resp.setRootOrgId(response.getRootOrgId());
		resp.setStudentId(response.getStudentId());
		return resp;
	}

}
