package cn.com.qmth.examcloud.exchange.outer.api.provider.swufe;

import io.swagger.annotations.ApiOperation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.common.dto.oe.marking.ExamRecordDto;
import cn.com.qmth.examcloud.exchange.base.enums.OrgIdServiceEnum;
import cn.com.qmth.examcloud.exchange.base.response.FailureBaseResponse;
import cn.com.qmth.examcloud.exchange.base.response.SuccessBaseResponse;
import cn.com.qmth.examcloud.exchange.base.util.SpringContextUtils;
import cn.com.qmth.examcloud.exchange.outer.service.ScorePushService;

/**
 * 推送成绩公共接口
 * @author  	chenken
 * @date    	2018年4月3日 下午3:57:34
 * @company 	QMTH
 * @description ScorePushController.java
 */
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/score-push")
@ConditionalOnProperty("app.pushscore.enable")
public class ScorePushController {
	
	private static final Logger log = LoggerFactory.getLogger(ScorePushController.class);
	
    @ApiOperation(value = "推送成绩")
    @PostMapping
    public ResponseEntity<Object> push(@RequestBody ExamRecordDto examRecordDto) throws Exception {
    	log.info("推送成绩orgId:"+examRecordDto.getOrgId()+"scoreId:"+examRecordDto.getScoreId());
    	OrgIdServiceEnum orgIdServiceEnum = OrgIdServiceEnum.getServiceNameByOrgId(examRecordDto.getOrgId());
    	if(orgIdServiceEnum !=null){
    		try{
    			ScorePushService scorePushService = (ScorePushService) SpringContextUtils.getBeanById(orgIdServiceEnum.getServiceName());
        		scorePushService.scorePush(examRecordDto);
        		return new ResponseEntity<>(new SuccessBaseResponse("分数ID:"+examRecordDto.getScoreId()+":推送成绩成功",null),HttpStatus.OK);
    		}catch(Exception e){
    			e.printStackTrace();
    			return new ResponseEntity<>(new FailureBaseResponse("分数ID:"+examRecordDto.getScoreId()+":推送成绩失败"),HttpStatus.INTERNAL_SERVER_ERROR);
    		}
    	}else{
    		return new ResponseEntity<>(new SuccessBaseResponse("没有机构ID为"+examRecordDto.getOrgId()+"的配置",null),HttpStatus.OK);
    	}
    }
}
