package cn.com.qmth.examcloud.exchange.outer.service;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.net.URI;

import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.HttpClientUtil;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentFaceCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentFaceInfoReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.exchange.dao.bean.FaceDetectResponse;
import cn.com.qmth.examcloud.exchange.outer.service.client.ExamStudentClient;
import cn.com.qmth.examcloud.exchange.outer.service.client.StudentFaceInfoClient;
import cn.com.qmth.examcloud.exchange.outer.service.impl.UpyunServiceImpl;

/**
 * Created by yuanpan on 2017/5/15.
 */
@Service
public class FaceppService {
	private static final Logger LOG = LoggerFactory.getLogger(FaceppService.class);

	@Value("${app.facepp.api_key}")
	private String apiKey;

	@Value("${app.facepp.api_secret}")
	private String apiSecret;

	@Autowired
	ExamStudentClient examStudentClient;

	@Autowired
	UpyunServiceImpl upyunService;

	@Autowired
	StudentFaceInfoClient studentFaceInfoClient;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	StudentFaceCloudService studentFaceCloudService;

	/**
	 * 重构<br>
	 * 调用人脸识别接口
	 *
	 * @author WANGWEI
	 * @param imageUrl
	 * @return
	 */
	public String detectSingleFace(String imageUrl) {
		// 重新检测次数为3次
		int time = 3;
		String faceToken = null;
		while (true) {
			if (time < 1) {
				throw new StatusException("EX--111010", "人脸检测失败");
			}

			CloseableHttpClient httpClient = HttpClients.createDefault();
			CloseableHttpResponse httpResponse = null;
			try {
				URI uri = new URIBuilder().setScheme("https").setHost("api-cn.faceplusplus.com")
						.setPath("/facepp/v3/detect").setParameter("api_key", apiKey)
						.setParameter("api_secret", apiSecret).setParameter("image_url", imageUrl)
						.build();
				HttpPost httpPost = new HttpPost(uri);

				httpResponse = httpClient.execute(httpPost);
				HttpEntity responseEntity = httpResponse.getEntity();
				int statusCode = httpResponse.getStatusLine().getStatusCode();
				BufferedReader reader = new BufferedReader(
						new InputStreamReader(responseEntity.getContent()));
				if (statusCode == 200) {
					Gson gson = new GsonBuilder().create();
					FaceDetectResponse faceDetectResponse = gson.fromJson(reader,
							FaceDetectResponse.class);
					if (faceDetectResponse.getFaces() != null
							&& faceDetectResponse.getFaces().size() > 0) {
						faceToken = faceDetectResponse.getFaces().get(0).getFace_token();
						break;
					} else {
						time--;
					}
				} else {
					StringBuffer buffer = new StringBuffer();
					String str = null;
					while (null != (str = reader.readLine())) {
						buffer.append(str);
					}
					String result = buffer.toString();
					LOG.error("获取token失败:" + result);
					time--;
				}
				httpClient.close();
			} catch (Exception e) {
				time--;
				LOG.error("人脸检测失败", e);
			} finally {
				HttpClientUtil.close(httpResponse);
			}
		}

		return faceToken;
	}

	/**
	 * 添加faceToken至faceSet
	 *
	 * @author WANGWEI
	 * @param faceToken
	 * @param facesetToken
	 */
	public void addFaceToken2FaceTokenSet(String faceToken, String facesetToken) {

		// 重新检测次数为3次
		int time = 3;
		while (time > 0) {
			// 调用人脸识别接口
			CloseableHttpClient httpClient = HttpClients.createDefault();
			CloseableHttpResponse httpResponse = null;
			try {
				URI uri = new URIBuilder().setScheme("https").setHost("api-cn.faceplusplus.com")
						.setPath("/facepp/v3/faceset/addface").setParameter("api_key", apiKey)
						.setParameter("api_secret", apiSecret)
						.setParameter("faceset_token", facesetToken)
						.setParameter("face_tokens", faceToken).build();

				HttpPost httpPost = new HttpPost(uri);
				httpResponse = httpClient.execute(httpPost);
				StatusLine statusLine = httpResponse.getStatusLine();
				LOG.info("response: " + EntityUtils.toString(httpResponse.getEntity()));
				if (statusLine.getStatusCode() == 200) {
					break;
				} else {
					time--;
				}
			} catch (Exception e) {
				LOG.error("添加faceToken至faceSet失败", e);
				time--;
			} finally {
				HttpClientUtil.close(httpResponse);
			}
		}
	}

	/**
	 * 重构
	 *
	 * @author WANGWEI
	 * @param accessUser
	 * @param fileName
	 * @param file
	 * @throws Exception
	 */
	public void uploadPhoto(User accessUser, String fileName, File file) throws Exception {
		Long rootOrgId = accessUser.getRootOrgId();
		String userToken = accessUser.getUserToken();
		if (!(fileName.endsWith(".jpg") || fileName.endsWith(".png"))) {
			throw new StatusException("B-101001", "文件格式错误");
		}

		String identityNumber = fileName.substring(0, fileName.lastIndexOf("."));

		GetStudentReq req = new GetStudentReq();
		req.setIdentityNumber(identityNumber);
		req.setRootOrgId(accessUser.getRootOrgId());
		GetStudentResp resp = studentCloudService.getStudent(req);
		StudentBean student = resp.getStudentInfo();

		String fileSuffix = fileName.substring(fileName.lastIndexOf("."));
		String newFileName = System.currentTimeMillis() + fileSuffix;
		String photoPath = upyunService.saveStudentPhoto(identityNumber, rootOrgId, newFileName,
				file);

		String url = upyunService.getStudentPhotoUrl(identityNumber, rootOrgId, newFileName);
		// 调用人脸识别接口
		String faceToken = detectSingleFace(url);

		String facesetToken = studentFaceInfoClient.getFaceSet(userToken);

		addFaceToken2FaceTokenSet(faceToken, facesetToken);

		SaveStudentFaceInfoReq saveStudentFaceInfoReq = new SaveStudentFaceInfoReq();
		saveStudentFaceInfoReq.setStudentId(student.getId());
		saveStudentFaceInfoReq.setPhotoPath(photoPath);
		saveStudentFaceInfoReq.setFaceToken(faceToken);
		saveStudentFaceInfoReq.setFaceSetToken(facesetToken);

		studentFaceCloudService.saveStudentFaceInfo(saveStudentFaceInfoReq);
	}

}
