package cn.com.qmth.examcloud.exchange.outer.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.em.enums.ExamType;
import cn.com.qmth.examcloud.exchange.base.exception.SaveExamException;
import cn.com.qmth.examcloud.exchange.dao.bean.ExamReq;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletExam;
import cn.com.qmth.examcloud.exchange.outer.service.ExamService;
import cn.com.qmth.examcloud.exchange.outer.service.client.ExamClient;

/**
 * @author  	chenken
 * @date    	2018年5月7日 下午3:29:39
 * @company 	QMTH
 * @description ExamServiceImpl.java
 */
@Service("examService")
public class ExamServiceImpl implements ExamService {

	@Autowired
	private ExamClient examClient;
	
	@Override
	public OutletExam createExam(ExamReq examReq,String userToken,Long rootOrgId) throws Exception {
		checkExamReq(examReq);
		OutletExam exam = new OutletExam();
        ResponseEntity<List<OutletExam>> examListResp = examClient.queryByName(userToken,examReq.getName());
        //判断考试是否存在
        if (examListResp != null && examListResp.getBody() != null && examListResp.getBody().size() > 0) {
            exam = examListResp.getBody().get(0);
        } else {
        	exam.setName(examReq.getName());
        	exam.setBeginTime(examReq.getBeginTime());
        	exam.setEndTime(examReq.getEndTime());
        	exam.setRemark(examReq.getName());//备注填考试名称
        	exam.setExamType(strToEnum(examReq.getExamType()));
        	exam.setDuration(examReq.getDuration());
        	exam.setEnable(true);		//默认开启考试
            exam.setOrgId(rootOrgId);	//设置 机构
            ResponseEntity<OutletExam> saveExamResp = examClient.save(userToken, exam);
            exam = saveExamResp.getBody();
        }
        exam.setAfterExamRemark(null);//考后说明置空
        exam.setBeforeExamRemark(null);////靠前说明置空
        return exam;
	}

	private void checkExamReq(ExamReq examReq) {
		if(examReq == null){
			throw new SaveExamException("考试信息不能为空");
		}
		if(examReq.getName() == null){
			throw new SaveExamException("考试名称不能为空");
		}
		if(examReq.getBeginTime() == null){
			throw new SaveExamException("考试开始时间不能为空");
		}
		if(examReq.getEndTime() == null){
			throw new SaveExamException("考试结束时间不能为空");
		}
	}
	
	public ExamType strToEnum(String str){
    	for(ExamType examType:ExamType.values()){
    		if(examType.name().equals(str)){
    			return examType;
    		}
    	}
    	return null;
    }

}

