package cn.com.qmth.examcloud.exchange.outer.service.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.em.ExamRecord;
import cn.com.qmth.examcloud.common.dto.em.ExamScore;
import cn.com.qmth.examcloud.common.dto.em.ExamStudent;
import cn.com.qmth.examcloud.common.dto.em.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.common.dto.em.enums.MarkingType;
import cn.com.qmth.examcloud.exchange.dao.ExamRecordDao;
import cn.com.qmth.examcloud.exchange.dao.ExamScoreDao;
import cn.com.qmth.examcloud.exchange.dao.OutletScoreDao;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletScore;
import cn.com.qmth.examcloud.exchange.outer.service.OutletScoreService;

/**
 * @author  	chenken
 * @date    	2018年1月3日 下午3:39:23
 * @company 	QMTH
 * @description OutletScoreServiceImpl.java
 */
@Service("outletScoreService")
public class OutletScoreServiceImpl implements OutletScoreService{

	@Autowired
	private OutletScoreDao outletScoreDao;
	
	@Autowired
	private ExamRecordDao examRecordDao;
	
	@Autowired
	private ExamScoreDao examScoreDao;
	
	@Override
	public List<OutletScore> getExamScoreById(Long scoreId)  throws Exception{
		if(scoreId == null){
			return null;
		}
		return outletScoreDao.getExamScoreById(scoreId);
	}
	
	@Override
	public List<OutletScore> queryExamScore(String examName, String examStuRemark,Long examOrgId) throws Exception {
		if(StringUtils.isBlank(examName) || examOrgId == null){
			return null;
		}
		List<ExamStudent> examStudents = outletScoreDao.findExamStudents(examName, examStuRemark, examOrgId);
		return getExamStudentsScore(examStudents);
	}

	@Override
	public List<OutletScore> queryExamScoreByIdentityNumbers(List<String> identityNumbers, Long examOrgId) throws Exception {
		if(identityNumbers==null||identityNumbers.size()==0||examOrgId == null){
			return null;
		}
		List<ExamStudent> examStudents = outletScoreDao.findExamStudents(identityNumbers,examOrgId);
		return getExamStudentsScore(examStudents);
	}
	
	@Override
	public List<OutletScore> queryExamScoreBy(Long orgId,String batchName,String courseCode, List<String> studentCodes) throws Exception {
		List<ExamStudent> examStudents = outletScoreDao.findExamStudents(orgId,batchName,courseCode,studentCodes);
		return getExamStudentsScore(examStudents);
	}
	
	private List<OutletScore> getExamStudentsScore(List<ExamStudent> examStudents) throws Exception{
		List<OutletScore> outletScoreList = new ArrayList<OutletScore>();
		
		for(ExamStudent examStudent:examStudents){
			OutletScore outletScore = new OutletScore(examStudent);
			outletScore = findExamData(outletScore,examStudent);
			outletScoreList.add(outletScore);
		}
		return outletScoreList;
	}
	
	private OutletScore findExamData(OutletScore outletScore,ExamStudent examStudent) throws Exception{
        List<ExamRecord> examRecordList = examRecordDao.findByExamStudentId(examStudent.getId());
        Boolean isFinished = examStudent.getFinished();
        if(isFinished==null){
        	isFinished = false;
        }
        //是否缺考
        outletScore.setIsMissExam(!isFinished);
        //是否违纪
        outletScore.setIsBreachThePrinciple(isBreachThePrinciple(isFinished,examRecordList));
        //最终成绩
        setFinalExamScoreAndEndTime(isFinished,outletScore,examRecordList,examStudent);
        return outletScore;
    }
	
	/**
	 * 1.设置最终成绩
	 * 2.设置考试结束时间
	 */
	public void setFinalExamScoreAndEndTime(Boolean isFinished,OutletScore outletScore,List<ExamRecord> examRecordList,ExamStudent examStudent){
		if(!isFinished){
			return;
		}
		ExamScore examScore = finalExamScore(examRecordList,examStudent.getExam().getMarkingType());
    	if(examScore != null){
    		outletScore.setScoreId(examScore.getId());
            outletScore.setTotalScore(examScore.getTotalScore());
            outletScore.setObjectiveScore(examScore.getObjectiveScore());
            outletScore.setSubjectiveScore(examScore.getSubjectiveScore());
            //设置考试结束时间和人脸识别成功率
            ExamRecord examRecord = examRecordDao.findByScoreId(examScore.getId());
            if(examRecord != null){
            	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            	Date endTime =  examRecord.getEndTime();
                if(endTime == null){
                	endTime = examRecord.getCleanTime();
                }
                outletScore.setEndTime(endTime == null?null:sdf.format(endTime));
            }
    	}
	}

	/**
	 * 查询并计算最终成绩
	 * @param isFinished
	 * @param examRecordList
	 * @param markingType
	 */
	private ExamScore finalExamScore(List<ExamRecord> examRecordAllList, MarkingType markingType) {
    	//第一次过滤考试记录:正常结束或者被系统处理的
    	List<ExamRecord> firstFilterExamRecordList = examRecordAllList.stream().filter(examRecord->{
    		return  examRecord.getStatus() == ExamRecordStatus.EXAM_END || 
    				examRecord.getStatus() == ExamRecordStatus.EXAM_OVERDUE;
    	}).collect(Collectors.toList());
    	
        if (firstFilterExamRecordList != null && firstFilterExamRecordList.size()>0) {
        	/**
        	 * 第二次过滤考试记录:过滤出有效的考试记录 
        	 * 1.没有警告的也没有违纪的
        	 * 2.有警告已审核通过的且没有违纪的
        	 */
            Stream<ExamRecord> secondFilterExamRecordStream = firstFilterExamRecordList.stream().filter(examRecord -> {
                return (!examRecord.getIsWarn() && examRecord.getInvigilatorOperation() != 1) 
                	|| (examRecord.getIsWarn() && examRecord.getIsAudit() && examRecord.getInvigilatorOperation() != 1);
            });
            
            List<ExamRecord> secondFilterExamRecords = secondFilterExamRecordStream.collect(Collectors.toList());
            if(secondFilterExamRecords == null|| secondFilterExamRecords.size() == 0){
            	return null;
            }
            //取出有效记录的成绩
            List<Long> examRecordIds = new ArrayList<Long>();
            for(int i = 0;i<secondFilterExamRecords.size();i++){
            	examRecordIds.add(secondFilterExamRecords.get(i).getId());
            }
            List<ExamScore> effectiveExamScoreList = examScoreDao.findByExamRecordIdIn(examRecordIds);
            
            //全部评阅规则或客观分最高规则:取总分最高
            if(markingType == MarkingType.ALL||markingType == MarkingType.OBJECT_SCORE_MAX){
				List<ExamScore> examScores = effectiveExamScoreList
											.stream()
											.sorted((o1,o2)->o2.getTotalScore().compareTo(o1.getTotalScore()))
											.collect(Collectors.toList());
				return examScores.get(0);
            }else if(markingType == MarkingType.LAST_SUBMIT){
            	//最后一次提交规则:取最后一次的成绩
            	List<ExamScore> examScores = effectiveExamScoreList
											.stream()
											.sorted((o1,o2)->o2.getId().compareTo(o1.getId()))
											.collect(Collectors.toList());
				return examScores.get(0);
			}else if(markingType == null){
				//离线考试
				if(effectiveExamScoreList != null&& effectiveExamScoreList.size()>0){
					return effectiveExamScoreList.get(0);
				}
			}
        }
        return null;
	}

	/**
	 * 是否违纪
	 * @param isFinished
	 * @param examRecordList
	 * @return
	 */
	private Boolean isBreachThePrinciple(Boolean isFinished,List<ExamRecord> examRecordList) {
		if(!isFinished){
    		return null;
    	}
		if(examRecordList==null || examRecordList.size()==0){
			return false;
		}
    	List<ExamRecord> examRecordFilterList = examRecordList.stream().filter(examRecord -> {
    		return  examRecord.getInvigilatorOperation() == 1;
    	}).collect(Collectors.toList());
        return (examRecordFilterList == null || examRecordFilterList.size()==0)?false:true;
	}

	@Override
	public List<OutletScore> queryExamScoreByExamNameAndIdentityNumbers(
			String examName, List<String> identityNumbers, Long examOrgId)
			throws Exception {
		if(StringUtils.isBlank(examName)){
			return null;
		}
		if(identityNumbers==null||identityNumbers.size()==0||examOrgId == null){
			return null;
		}
		List<ExamStudent> examStudents = outletScoreDao.findExamStudents(examName,identityNumbers,examOrgId);
		return getExamStudentsScore(examStudents);
	}

	@Override
	public List<OutletScore> queryExamScoreBy(Long rootOrgId, String examName,
			String studentCode, String courseCode) throws Exception {
		if(rootOrgId == null){
			return null;
		}
		if(examName == null){
			return null;
		}
		if(studentCode == null){
			return null;
		}
		if(courseCode == null){
			return null;
		}
		return outletScoreDao.queryExamScoreBy(rootOrgId, examName, studentCode, courseCode);
	}

}

