package cn.com.qmth.examcloud.exchange.outer.service.impl;

import java.util.Date;
import java.util.HashSet;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.core.User;
import cn.com.qmth.examcloud.common.dto.core.enums.CourseLevel;
import cn.com.qmth.examcloud.exchange.base.enums.OutletOrgType;
import cn.com.qmth.examcloud.exchange.base.exception.NoSuchCourseLevelException;
import cn.com.qmth.examcloud.exchange.base.exception.SaveExamStudentException;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletCourse;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletExam;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletExamStudent;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletOrg;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletStudent;
import cn.com.qmth.examcloud.exchange.dao.bean.StudentInfoReq;
import cn.com.qmth.examcloud.exchange.outer.service.StudentInfoService;
import cn.com.qmth.examcloud.exchange.outer.service.client.CourseClient;
import cn.com.qmth.examcloud.exchange.outer.service.client.ExamClient;
import cn.com.qmth.examcloud.exchange.outer.service.client.ExamStudentClient;
import cn.com.qmth.examcloud.exchange.outer.service.client.OrgClient;
import cn.com.qmth.examcloud.exchange.outer.service.client.StudentClient;

/**
 * @author  	chenken
 * @date    	2018年5月7日 下午4:33:27
 * @company 	QMTH
 * @description StudentInfoServiceImpl.java
 */
@Service("studentInfoService")
public class StudentInfoServiceImpl implements StudentInfoService {

	private static final Logger log = LoggerFactory.getLogger(StudentInfoServiceImpl.class);
	
    @Autowired
    private OrgClient orgClient;

    @Autowired
    private ExamClient examClient;

    @Autowired
    private CourseClient courseClient;

    @Autowired
    private StudentClient studentClient;

    @Autowired
    private ExamStudentClient examStudentClient;
	
	@Override
	public void saveStudentInfo(List<StudentInfoReq> studentInfoReqs,String userToken,Long rootOrgId) throws Exception {
		log.info("创建考试和导入学生开始");
		OutletExam exam = chechExamStudentInfo(studentInfoReqs,rootOrgId,userToken);
        for (StudentInfoReq studentInfoReq : studentInfoReqs) {
        	OutletOrg org = saveOrg(studentInfoReq,userToken,rootOrgId);
            
        	OutletCourse course = saveCourse(studentInfoReq,userToken,rootOrgId);
        	
        	OutletStudent student = saveStudent(studentInfoReq,org,userToken,rootOrgId);
        	
        	saveExamStudent(studentInfoReq,student,course,org,exam,userToken,rootOrgId);
        }
        log.info("创建考试和导入学生完成");
	}

	/**
     * 保存学习中心
     * @param studentInfoReq
     * @param userToken
     * @param rootOrgId
     * @return
     */
    private OutletOrg saveOrg(StudentInfoReq studentInfoReq,String userToken,Long rootOrgId){
    	log.info("2.保存学习中心,学习中心code:"+studentInfoReq.getOrgCode());
    	//判断学习中心是否存在，不存在则创建
        String orgCode = studentInfoReq.getOrgCode();
        //根据orgCode，查询学习中心
        ResponseEntity<OutletOrg> orgResponseEntity = orgClient.findByParentIdAndCode(userToken, rootOrgId, orgCode);
        OutletOrg org = orgResponseEntity.getBody();
        if (org == null) {
            org = new OutletOrg();
            org.setName(studentInfoReq.getOrgName());
            org.setCode(studentInfoReq.getOrgCode());
            org.setParentId(rootOrgId);
            org.setRootId(rootOrgId);
            org.setEnable(true);
            org.setType(OutletOrgType.SCHOOL);
            ResponseEntity<OutletOrg> saveOrgResp = orgClient.save(userToken, org);
            org = saveOrgResp.getBody();
        }
        return org;
    }
    
    /**
     * 保存课程
     * @param studentInfoReq
     * @param userToken
     * @param rootOrgId
     * @throws NoSuchCourseLevelException
     */
    private OutletCourse saveCourse(StudentInfoReq studentInfoReq,String userToken,Long rootOrgId) throws NoSuchCourseLevelException{
    	//判断课程是否存在，不存在则新增
    	String courseCode = studentInfoReq.getCourseCode();
    	if(StringUtils.isNotBlank(studentInfoReq.getCourseLevel())){
    		CourseLevel courseLevel = getCourseLevelByCName(studentInfoReq.getCourseLevel());
    		if(courseLevel!=null){
    			courseCode = courseLevel.getAbbreviate()+studentInfoReq.getCourseCode();
    		}
    	}
    	
        log.info("3.保存课程,课程code:"+courseCode);
        studentInfoReq.setCourseCode(courseCode);
        ResponseEntity<OutletCourse> getCourseResp = courseClient.getCourseByCode(userToken, rootOrgId, studentInfoReq.getCourseCode());
        OutletCourse course = getCourseResp.getBody();
        if (course == null) {
            course = new OutletCourse();
            course.setOrgId(rootOrgId);
            course.setEnable(true);
            course.setCode(studentInfoReq.getCourseCode());
            course.setCreateTime(new Date());
            if(StringUtils.isNotBlank(studentInfoReq.getCourseLevel())){
            	course.setLevel(getCourseLevelByCName(studentInfoReq.getCourseLevel()));
            }
            course.setName(studentInfoReq.getCourseName());
            ResponseEntity<OutletCourse> saveCourseResp = courseClient.save(userToken, course);
            course = saveCourseResp.getBody();
        }
        return course;
    }
    
    /**
     * 保存学生信息
     * @param studentInfoReq
     * @param org
     * @param userToken
     * @param rootOrgId
     * @return
     */
    private OutletStudent saveStudent(StudentInfoReq studentInfoReq,OutletOrg org,String userToken,Long rootOrgId) {
    	//保存学生信息，在保存学生信息的接口中会自动保存用户信息，所以不需要单独调用UserApi去保存用户信息
    	log.info("4.保存学生信息,学生身份证号:"+studentInfoReq.getIdentityNumber());
    	OutletStudent student = new OutletStudent();
        student.setIdentityNumber(studentInfoReq.getIdentityNumber());
        student.setName(studentInfoReq.getStudentName());
        student.setStudentCode(studentInfoReq.getStudentCode());
        User user = new User();
        user.setOrgId(org.getId());
        user.setRootOrgId(rootOrgId);
        user.setEnable(true);
        student.setUser(user);
        //查询学生信息是否存在
        List<OutletStudent> studentList = studentClient.getStudentByIdentityNumber(userToken, studentInfoReq.getIdentityNumber()).getBody();
        if (studentList == null || studentList.size() < 1) {
        	student.setCreateTime(new Date());
            ResponseEntity<OutletStudent> saveStudentResp = studentClient.save(userToken, student);
            student = saveStudentResp.getBody();
        } else {
        	OutletStudent stu = studentList.get(0);
            student.setOrgId(org.getId());
            student.setRootOrgId(rootOrgId);
            student.setId(stu.getId());
            student.setUser(stu.getUser());
            ResponseEntity<OutletStudent> updateStudentResp = studentClient.updateStudent(userToken,student);
            student = updateStudentResp.getBody();
        }
        return student;
	}
    
    /**
     * 保存考生信息
     * @param studentInfoReq
     * @param student
     * @param org
     * @param exam
     * @param userToken
     * @param rootOrgId
     * @throws NoSuchCourseLevelException
     * @throws SaveExamStudentException
     */
	private void saveExamStudent(StudentInfoReq studentInfoReq,OutletStudent student,OutletCourse course,OutletOrg org,OutletExam exam,String userToken,Long rootOrgId) throws NoSuchCourseLevelException, SaveExamStudentException {
		log.info("5.保存考生信息,考试ID:"+exam.getId()
							  +"课程code:"+studentInfoReq.getCourseCode()
							  +",考生身份证号:"+studentInfoReq.getIdentityNumber());
		//判断考生信息是否存在
        ResponseEntity<List<OutletExamStudent>> examStudentListResp = examStudentClient.queryExamStudent(userToken, studentInfoReq.getIdentityNumber(), exam.getId(), studentInfoReq.getCourseCode());
        if (examStudentListResp != null && examStudentListResp.getBody() != null && examStudentListResp.getBody().size() > 0) {
        	//学生该门课程考试已经存在
        	List<OutletExamStudent> examStudents = examStudentListResp.getBody();
        	OutletExamStudent examStudentOld = examStudents.get(0);
        	//更新相关的姓名、学习中心、专业
        	examStudentOld.setName(student.getName());
        	examStudentOld.setOrgId(org.getId());
        	examStudentOld.setOrgCode(org.getCode());
        	examStudentOld.setOrgName(org.getName());
        	examStudentOld.setRootOrgId(rootOrgId);
        	examStudentOld.setSpecialtyName(studentInfoReq.getSpecialtyName());
        	 try {
                 examStudentClient.save(userToken, examStudentOld);
             } catch (Exception e) {
                 throw new SaveExamStudentException("保存考生失败");
             }
        } else {
        	//考生不存在
        	OutletExamStudent examStudent =  initExamStudent(studentInfoReq,student,org,exam,course,rootOrgId);
            try {
                examStudentClient.save(userToken, examStudent);
            } catch (Exception e) {
                throw new SaveExamStudentException("保存考生失败");
            }
        }
	}
    
    private OutletExamStudent initExamStudent(StudentInfoReq studentInfoReq,OutletStudent student, OutletOrg org, OutletExam exam,OutletCourse course,Long rootOrgId){
    	OutletExamStudent examStudent = new OutletExamStudent();
    	examStudent.setName(studentInfoReq.getStudentName());
    	examStudent.setStudentCode(studentInfoReq.getStudentCode());
    	examStudent.setIdentityNumber(studentInfoReq.getIdentityNumber());
    	examStudent.setSpecialtyName(studentInfoReq.getSpecialtyName());
    	examStudent.setPaperType(studentInfoReq.getPaperType());
    	examStudent.setInfoCollector(studentInfoReq.getInfoCollector());
    	examStudent.setRemark(studentInfoReq.getRemark());
    	examStudent.setGrade(studentInfoReq.getGrade());
    	examStudent.setGraduated(studentInfoReq.getGraduated());
    	
    	examStudent.setStudentId(student.getId());
    	
    	examStudent.setOrgId(org.getId());
    	examStudent.setOrgCode(org.getCode());
    	examStudent.setOrgName(org.getName());
    	examStudent.setRootOrgId(rootOrgId);
    	examStudent.setExam(exam);
    	
    	examStudent.setCourseCode(course.getCode());
    	examStudent.setCourseName(course.getName());
    	if(StringUtils.isNotBlank(studentInfoReq.getCourseLevel())){
    		examStudent.setCourseLevel(getCourseLevelByCName(studentInfoReq.getCourseLevel())+"");
    	}
    	examStudent.setFinished(false);
		return examStudent;
	}

    private CourseLevel getCourseLevelByCName(String cname){
    	if(StringUtils.isBlank(cname)){
    		return null;
    	}
        for(CourseLevel level:CourseLevel.values()){
        	if(cname.equals(level.getName())){
        		return level;
        	}
        }
        return null;
    }

	/**
	 * 检查数据
	 * @param examStudentReqList
	 */
	private OutletExam chechExamStudentInfo(List<StudentInfoReq> examStudentReqList, Long rootOrgId,String userToken){
		if(examStudentReqList==null||examStudentReqList.size()==0){
			throw new SaveExamStudentException("集合不能为空");
		}
		if(examStudentReqList.size()>1000){
			throw new SaveExamStudentException("单次推送集合长度不能大于1000");
		}
		HashSet<String> examNameSet = new HashSet<String>();
		for(StudentInfoReq examStudentAllInfoReq:examStudentReqList){
			examNameSet.add(examStudentAllInfoReq.getExamName());
		}
		if(examNameSet.size()>1){
			throw new SaveExamStudentException("集合中不允许存在多个考试");
		}
		String examName = examStudentReqList.get(0).getExamName();
		ResponseEntity<List<OutletExam>> examListResp = examClient.queryByName(userToken, examName);
		
		if (examListResp == null || examListResp.getBody() == null || examListResp.getBody().size() == 0) {
			throw new SaveExamStudentException("考试不存在,请先创建考试");
		}
		
		if(examListResp.getBody().size()>1){
			throw new SaveExamStudentException("当前机构中存在多个名称为"+examName+"的考试,请重新创建考试");
		}
		
		return examListResp.getBody().get(0);
	}
	
}

