package cn.com.qmth.examcloud.exchange.outer.service.impl;

import java.io.File;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.UrlUtil;
import cn.com.qmth.examcloud.exchange.outer.service.UpyunService;
import main.java.com.UpYun;

/**
 * Created by yuanpan on 2017/5/11.
 */
@Service
public class UpyunServiceImpl implements UpyunService {

	private final Object LOCK = new Object();

	private UpYun upyun;

	@Value("${app.upyun.path}")
	private String path;

	@Value("${app.upyun.bucket}")
	private String bucket;

	@Value("${app.upyun.operator}")
	private String operator;

	@Value("${app.upyun.password}")
	private String password;

	private UpYun getUpyun() {
		// modified by wangwei.
		if (null == upyun) {
			synchronized (LOCK) {
				if (null == upyun) {
					upyun = new UpYun(bucket, operator, password);
				}
			}
		}
		return upyun;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param identityNumber
	 * @param rootOrgId
	 * @param fileName
	 * @return
	 */
	public String getStudentPhotoUrl(String identityNumber, Long rootOrgId, String fileName) {
		identityNumber = UrlUtil.encode(identityNumber);
		String url = "http://" + bucket + ".b0.upaiyun.com" + path + "/" + rootOrgId + "/"
				+ identityNumber + "/" + fileName;
		return url;
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see cn.com.qmth.examcloud.exchange.outer.service.UpyunService#
	 * saveStudentPhoto(java.lang.String, java.lang.Long, java.lang.String,
	 * java.io.File)
	 */
	@Override
	public String saveStudentPhoto(String identityNumber, Long rootOrgId, String fileName,
			File file) {
		identityNumber = UrlUtil.encode(identityNumber);
		String filePath = path + "/" + rootOrgId + "/" + identityNumber + "/" + fileName;
		try {
			getUpyun().writeFile(filePath, file, true);
			return rootOrgId + "/" + identityNumber + "/" + fileName;
		} catch (Exception e) {
			throw new StatusException("EX-100001", "调用又拍云API写文件失败");
		}
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see cn.com.qmth.examcloud.exchange.outer.service.UpyunService#
	 * saveStudentPhoto(java.lang.String, java.lang.Long, java.lang.String,
	 * byte[])
	 */
	@Override
	public String saveStudentPhoto(String identityNumber, Long rootOrgId, String fileName,
			byte[] bytes) {
		String filePath = path + "/" + rootOrgId + "/" + identityNumber + "/" + fileName;
		try {
			getUpyun().writeFile(filePath, bytes, true);
			return rootOrgId + "/" + identityNumber + "/" + fileName;
		} catch (Exception e) {
			throw new StatusException("EX-100001", "调用又拍云API写文件失败");
		}
	}
}
