package cn.com.qmth.examcloud.exchange.outer.service.impl.scorePush;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.oe.marking.ExamRecordDto;
import cn.com.qmth.examcloud.common.dto.oe.marking.PushScoreActionEnum;
import cn.com.qmth.examcloud.exchange.base.util.OutletUtils;
import cn.com.qmth.examcloud.exchange.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletScore;
import cn.com.qmth.examcloud.exchange.outer.service.OutletScoreService;
import cn.com.qmth.examcloud.exchange.outer.service.ScorePushService;

/**
 * @author  	chenken
 * @date    	2018年5月17日 下午2:22:43
 * @company 	QMTH
 * @description 西南财大推送成绩接口
 */
@Service("xncdScorePushService")
public class XncdScorePushServiceImpl  implements ScorePushService{

	private static final Logger log = LoggerFactory.getLogger(XncdScorePushServiceImpl.class);
	
    @Autowired
    private ExamScoreRepo examScoreRepo;
    
    @Autowired
    private OutletScoreService outletScoreService;
    
    private static final String pushScoreUrl = "http://www.swufe-online.com/netedu/qmth.do";
	
	@Override
	public void scorePush(ExamRecordDto examRecordDto) throws Exception {
		//西南财大只有考试结束后或自动清理才推送分数
		if(examRecordDto.getPushScoreActionEnum() == PushScoreActionEnum.END_EXAM 
				|| examRecordDto.getPushScoreActionEnum() == PushScoreActionEnum.CLEAN){
			log.info("开始推送西南财大考生成绩 scoreId:"+examRecordDto.getScoreId());
			Long scoreId = examRecordDto.getScoreId();
	        List<OutletScore> outletScores = outletScoreService.getExamScoreById(scoreId);
	        if (outletScores != null && outletScores.size() > 0) {
	        	OutletScore outletScore = outletScores.get(0);
	        	
	        	CloseableHttpResponse httpResponse = null;
	            CloseableHttpClient httpClient = HttpClients.createDefault();;
	            HttpPost httpPost = new HttpPost(pushScoreUrl);
	            
	            List<NameValuePair> params = buildParamsData(outletScore);
	            
	            httpPost.setEntity(new UrlEncodedFormEntity(params,"UTF-8"));
	            
	            httpResponse = httpClient.execute(httpPost);
	            
	            String result = "";
	            HttpEntity responseEntity = httpResponse.getEntity();
	            int statusCode= httpResponse.getStatusLine().getStatusCode();
	            if(statusCode == 200){
	                BufferedReader reader = new BufferedReader(new InputStreamReader(responseEntity.getContent()));
	                StringBuffer buffer = new StringBuffer();
	                String str = "";
	                while(StringUtils.isNoneBlank((str = reader.readLine()))) {
	                    buffer.append(str);
	                }
	                result = buffer.toString();
	            }
	                 
	            httpClient.close();
	            if(httpResponse!=null){
	                httpResponse.close();
	            }
	            log.info(result);
	        }
		}
	}
	
	private List<NameValuePair> buildParamsData(OutletScore outletScore){
		Map<String,Object> map = OutletUtils.getKeyAndValue(outletScore);
        map.remove("examRemark");//西南财大接口不需要
        map.remove("studentRemark");
        map.remove("isMissExam");
        
        map.put("op","pushScore");
        map.put("x","xczx_white_list");
        map.put("LoginName","qmth");
        map.put("PassWord","qmth123!");
        
        //是否违纪
        boolean isBreachThePrinciple = (boolean) map.get("isBreachThePrinciple");
        map.put("isBreachThePrinciple", isBreachThePrinciple?1:0);
        
        List<NameValuePair> params = new ArrayList<NameValuePair>();
        for(String key:map.keySet()){
        	params.add(new BasicNameValuePair(key,map.get(key)+""));
        }
        
        return params;
	}

}

